
def load_data():
    from . import __get_file
    base = 'https://www2.informatik.hu-berlin.de/~naoth/ressources/log/demo_edgels/'
    file = ('2019-07-05_11-45-00_Berlin United_vs_NomadZ_half2-1_93_Nao0212.log', 'e3f0c5fde010560cafd22c7d1e36cef7')
    return __get_file(file[0], file_hash=file[1], origin=base, cache_subdir='demo_edgels')



try:
    from . import AudioData_pb2
    from . import CommonTypes_pb2
    from . import Framework_Representations_pb2
    from . import Messages_pb2
    from . import Representations_pb2
    from . import TeamMessage_pb2
    from . import RobotPose_pb2
except ImportError as e:
    print("\n\033[91mThe protobuf files can not be found/imported! Please run '\033[1mpremake --protoc\033[0m\033[91m' first.\033[0m\n")
    raise e




class XABSLSymbols:
    def __init__(self):
        self.values = {}
        self.decimalIdToName = {}
        self.booleanIdToName = {}
        self.enumIdToName = {}

    def __get_or_set(self, var, id, value):
        if value is None:
            if isinstance(id, str):
                return self.values[id]
            else:
                return self.values[var[id]]
        else:
            if isinstance(id, str):
                self.values[id] = value
            else:
                self.values[var[id]] = value

    def decimal(self, id, value=None):
        return self.__get_or_set(self.decimalIdToName, id, value)

    def boolean(self, id, value=None):
        return self.__get_or_set(self.booleanIdToName, id, value)

    def enum(self, id, value=None):
        return self.__get_or_set(self.enumIdToName, id, value)

    def __str_item(self, name, item):
        result = name + ' = {\n'
        for s in item:
            result += '\t{:3d}, {} = {}\n'.format(s, item[s], self.values[item[s]])
        result += '}'
        return result

    def __str__(self):
        result = self.__str_item('DecimalSymbols', self.decimalIdToName) \
               + ',\n' + self.__str_item('BooleanSymbols', self.booleanIdToName) \
               + ',\n' + self.__str_item('EnumSymbols', self.enumIdToName)
        return result



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TeamMessage.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import CommonTypes_pb2 as CommonTypes__pb2
from . import Framework_Representations_pb2 as Framework__Representations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TeamMessage.proto',
  package='naothmessages',
  syntax='proto2',
  serialized_pb=_b('\n\x11TeamMessage.proto\x12\rnaothmessages\x1a\x11\x43ommonTypes.proto\x1a\x1f\x46ramework-Representations.proto\"\x90\x01\n\tRobotRole\x12\x41\n\x0brole_static\x18\x01 \x02(\x0e\x32\x1e.naothmessages.RobotRoleStatic:\x0crole_unknown\x12@\n\x0crole_dynamic\x18\x02 \x02(\x0e\x32\x1f.naothmessages.RobotRoleDynamic:\trole_none\"L\n\x08Opponent\x12\x14\n\tplayerNum\x18\x01 \x02(\x05:\x01\x30\x12*\n\x0bposeOnField\x18\x02 \x01(\x0b\x32\x15.naothmessages.Pose2D\"A\n\x03Ntp\x12\x14\n\tplayerNum\x18\x01 \x02(\x05:\x01\x30\x12\x0f\n\x04sent\x18\x02 \x01(\x04:\x01\x30\x12\x13\n\x08received\x18\x03 \x01(\x04:\x01\x30\"\x8e\x01\n\x05\x44rops\x12\x1b\n\x10\x64ropNoSplMessage\x18\x01 \x01(\r:\x01\x30\x12\x19\n\x0e\x64ropNotOurTeam\x18\x02 \x01(\r:\x01\x30\x12\x1b\n\x10\x64ropNotParseable\x18\x03 \x01(\r:\x01\x30\x12\x16\n\x0b\x64ropKeyFail\x18\x04 \x01(\r:\x01\x30\x12\x18\n\rdropMonotonic\x18\x05 \x01(\r:\x01\x30\"\x98\x05\n\x11\x42UUserTeamMessage\x12\x17\n\x06\x62odyID\x18\x01 \x01(\t:\x07unknown\x12\x12\n\ntimeToBall\x18\x02 \x01(\r\x12\x19\n\nwasStriker\x18\x03 \x01(\x08:\x05\x66\x61lse\x12\x1e\n\x0bisPenalized\x18\x04 \x01(\x08:\x05\x66\x61lseB\x02\x18\x01\x12*\n\topponents\x18\x05 \x03(\x0b\x32\x17.naothmessages.Opponent\x12\x19\n\nteamNumber\x18\x06 \x01(\r:\x01\x30\x42\x02\x18\x01\x12\x18\n\rbatteryCharge\x18\x07 \x01(\x02:\x01\x31\x12\x16\n\x0btemperature\x18\x08 \x01(\x02:\x01\x30\x12\x14\n\ttimestamp\x18\t \x01(\x04:\x01\x30\x12\x1f\n\x10wantsToBeStriker\x18\n \x01(\x08:\x05\x66\x61lse\x12\x19\n\x0e\x63puTemperature\x18\x0b \x01(\x02:\x01\x30\x12\x1e\n\x0fwhistleDetected\x18\x0c \x01(\x08:\x05\x66\x61lse\x12\x17\n\x0cwhistleCount\x18\r \x01(\x05:\x01\x30\x12.\n\x08teamBall\x18\x0e \x01(\x0b\x32\x1c.naothmessages.DoubleVector2\x12&\n\nntpRequest\x18\x0f \x03(\x0b\x32\x12.naothmessages.Ntp\x12\x32\n\x0c\x62\x61llVelocity\x18\x10 \x01(\x0b\x32\x1c.naothmessages.DoubleVector2\x12\x36\n\nrobotState\x18\x11 \x01(\x0e\x32\x19.naothmessages.RobotState:\x07initial\x12+\n\trobotRole\x18\x12 \x01(\x0b\x32\x18.naothmessages.RobotRole\x12\x13\n\x0breadyToWalk\x18\x13 \x01(\x08\x12\x11\n\x03key\x18\x64 \x01(\t:\x04none\"\xee\x03\n\x0bTeamMessage\x12-\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x1f.naothmessages.TeamMessage.Data\x12)\n\x0bmessageDrop\x18\x02 \x01(\x0b\x32\x14.naothmessages.Drops\x1a\x84\x03\n\x04\x44\x61ta\x12\x14\n\tplayerNum\x18\x01 \x01(\r:\x01\x30\x12\x39\n\tteamColor\x18\x0b \x01(\x0e\x32\x18.naothmessages.TeamColor:\x08\x62lueTeamB\x02\x18\x01\x12#\n\x04pose\x18\x03 \x01(\x0b\x32\x15.naothmessages.Pose2D\x12\x13\n\x07\x62\x61llAge\x18\x04 \x01(\x05:\x02-1\x12\x32\n\x0c\x62\x61llPosition\x18\x05 \x01(\x0b\x32\x1c.naothmessages.DoubleVector2\x12\x32\n\x0c\x62\x61llVelocity\x18\x06 \x01(\x0b\x32\x1c.naothmessages.DoubleVector2\x12\x15\n\x06\x66\x61llen\x18\n \x01(\x08:\x05\x66\x61lse\x12.\n\x04user\x18\x08 \x01(\x0b\x32 .naothmessages.BUUserTeamMessage\x12+\n\tframeInfo\x18\t \x01(\x0b\x32\x18.naothmessages.FrameInfo\x12\x15\n\nteamNumber\x18\x0c \x01(\r:\x01\x30*\xb2\x01\n\tTeamColor\x12\x0c\n\x08\x62lueTeam\x10\x00\x12\x0b\n\x07redTeam\x10\x01\x12\x0e\n\nyellowTeam\x10\x02\x12\r\n\tblackTeam\x10\x03\x12\r\n\twhiteTeam\x10\x04\x12\r\n\tgreenTeam\x10\x05\x12\x0e\n\norangeTeam\x10\x06\x12\x0e\n\npurpleTeam\x10\x07\x12\r\n\tbrownTeam\x10\x08\x12\x0c\n\x08grayTeam\x10\t\x12\x10\n\x0binvalidTeam\x10\xff\x01*d\n\nRobotState\x12\x0b\n\x07initial\x10\x00\x12\t\n\x05ready\x10\x01\x12\x07\n\x03set\x10\x02\x12\x0b\n\x07playing\x10\x03\x12\x0c\n\x08\x66inished\x10\x04\x12\r\n\tpenalized\x10\x05\x12\x0b\n\x07unstiff\x10\x06*\x98\x02\n\x0fRobotRoleStatic\x12\x16\n\x12role_defender_left\x10\x00\x12\x18\n\x14role_defender_center\x10\x01\x12\x17\n\x13role_defender_right\x10\x02\x12\x18\n\x14role_midfielder_left\x10\x03\x12\x1a\n\x16role_midfielder_center\x10\x04\x12\x19\n\x15role_midfielder_right\x10\x05\x12\x15\n\x11role_forward_left\x10\x06\x12\x17\n\x13role_forward_center\x10\x07\x12\x16\n\x12role_forward_right\x10\x08\x12\x0f\n\x0brole_goalie\x10\t\x12\x10\n\x0crole_unknown\x10\n*b\n\x10RobotRoleDynamic\x12\r\n\trole_none\x10\x00\x12\x12\n\x0erole_supporter\x10\x01\x12\x19\n\x15role_goalie_supporter\x10\x02\x12\x10\n\x0crole_striker\x10\x03\x42\x1b\n\x19\x64\x65.naoth.rc.core.messages')
  ,
  dependencies=[CommonTypes__pb2.DESCRIPTOR,Framework__Representations__pb2.DESCRIPTOR,])

_TEAMCOLOR = _descriptor.EnumDescriptor(
  name='TeamColor',
  full_name='naothmessages.TeamColor',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='blueTeam', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='redTeam', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='yellowTeam', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='blackTeam', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='whiteTeam', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='greenTeam', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='orangeTeam', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='purpleTeam', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='brownTeam', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='grayTeam', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='invalidTeam', index=10, number=255,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1690,
  serialized_end=1868,
)
_sym_db.RegisterEnumDescriptor(_TEAMCOLOR)

TeamColor = enum_type_wrapper.EnumTypeWrapper(_TEAMCOLOR)
_ROBOTSTATE = _descriptor.EnumDescriptor(
  name='RobotState',
  full_name='naothmessages.RobotState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='initial', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ready', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='set', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='playing', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='finished', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='penalized', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='unstiff', index=6, number=6,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1870,
  serialized_end=1970,
)
_sym_db.RegisterEnumDescriptor(_ROBOTSTATE)

RobotState = enum_type_wrapper.EnumTypeWrapper(_ROBOTSTATE)
_ROBOTROLESTATIC = _descriptor.EnumDescriptor(
  name='RobotRoleStatic',
  full_name='naothmessages.RobotRoleStatic',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='role_defender_left', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_defender_center', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_defender_right', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_midfielder_left', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_midfielder_center', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_midfielder_right', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_forward_left', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_forward_center', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_forward_right', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_goalie', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_unknown', index=10, number=10,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1973,
  serialized_end=2253,
)
_sym_db.RegisterEnumDescriptor(_ROBOTROLESTATIC)

RobotRoleStatic = enum_type_wrapper.EnumTypeWrapper(_ROBOTROLESTATIC)
_ROBOTROLEDYNAMIC = _descriptor.EnumDescriptor(
  name='RobotRoleDynamic',
  full_name='naothmessages.RobotRoleDynamic',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='role_none', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_supporter', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_goalie_supporter', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='role_striker', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2255,
  serialized_end=2353,
)
_sym_db.RegisterEnumDescriptor(_ROBOTROLEDYNAMIC)

RobotRoleDynamic = enum_type_wrapper.EnumTypeWrapper(_ROBOTROLEDYNAMIC)
blueTeam = 0
redTeam = 1
yellowTeam = 2
blackTeam = 3
whiteTeam = 4
greenTeam = 5
orangeTeam = 6
purpleTeam = 7
brownTeam = 8
grayTeam = 9
invalidTeam = 255
initial = 0
ready = 1
set = 2
playing = 3
finished = 4
penalized = 5
unstiff = 6
role_defender_left = 0
role_defender_center = 1
role_defender_right = 2
role_midfielder_left = 3
role_midfielder_center = 4
role_midfielder_right = 5
role_forward_left = 6
role_forward_center = 7
role_forward_right = 8
role_goalie = 9
role_unknown = 10
role_none = 0
role_supporter = 1
role_goalie_supporter = 2
role_striker = 3



_ROBOTROLE = _descriptor.Descriptor(
  name='RobotRole',
  full_name='naothmessages.RobotRole',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role_static', full_name='naothmessages.RobotRole.role_static', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=10,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='role_dynamic', full_name='naothmessages.RobotRole.role_dynamic', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=89,
  serialized_end=233,
)


_OPPONENT = _descriptor.Descriptor(
  name='Opponent',
  full_name='naothmessages.Opponent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='playerNum', full_name='naothmessages.Opponent.playerNum', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='poseOnField', full_name='naothmessages.Opponent.poseOnField', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=235,
  serialized_end=311,
)


_NTP = _descriptor.Descriptor(
  name='Ntp',
  full_name='naothmessages.Ntp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='playerNum', full_name='naothmessages.Ntp.playerNum', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sent', full_name='naothmessages.Ntp.sent', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='received', full_name='naothmessages.Ntp.received', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=313,
  serialized_end=378,
)


_DROPS = _descriptor.Descriptor(
  name='Drops',
  full_name='naothmessages.Drops',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dropNoSplMessage', full_name='naothmessages.Drops.dropNoSplMessage', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropNotOurTeam', full_name='naothmessages.Drops.dropNotOurTeam', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropNotParseable', full_name='naothmessages.Drops.dropNotParseable', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropKeyFail', full_name='naothmessages.Drops.dropKeyFail', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropMonotonic', full_name='naothmessages.Drops.dropMonotonic', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=381,
  serialized_end=523,
)


_BUUSERTEAMMESSAGE = _descriptor.Descriptor(
  name='BUUserTeamMessage',
  full_name='naothmessages.BUUserTeamMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bodyID', full_name='naothmessages.BUUserTeamMessage.bodyID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("unknown").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeToBall', full_name='naothmessages.BUUserTeamMessage.timeToBall', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='wasStriker', full_name='naothmessages.BUUserTeamMessage.wasStriker', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='isPenalized', full_name='naothmessages.BUUserTeamMessage.isPenalized', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='opponents', full_name='naothmessages.BUUserTeamMessage.opponents', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='teamNumber', full_name='naothmessages.BUUserTeamMessage.teamNumber', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='batteryCharge', full_name='naothmessages.BUUserTeamMessage.batteryCharge', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='temperature', full_name='naothmessages.BUUserTeamMessage.temperature', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='naothmessages.BUUserTeamMessage.timestamp', index=8,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='wantsToBeStriker', full_name='naothmessages.BUUserTeamMessage.wantsToBeStriker', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cpuTemperature', full_name='naothmessages.BUUserTeamMessage.cpuTemperature', index=10,
      number=11, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='whistleDetected', full_name='naothmessages.BUUserTeamMessage.whistleDetected', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='whistleCount', full_name='naothmessages.BUUserTeamMessage.whistleCount', index=12,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='teamBall', full_name='naothmessages.BUUserTeamMessage.teamBall', index=13,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ntpRequest', full_name='naothmessages.BUUserTeamMessage.ntpRequest', index=14,
      number=15, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ballVelocity', full_name='naothmessages.BUUserTeamMessage.ballVelocity', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='robotState', full_name='naothmessages.BUUserTeamMessage.robotState', index=16,
      number=17, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='robotRole', full_name='naothmessages.BUUserTeamMessage.robotRole', index=17,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='readyToWalk', full_name='naothmessages.BUUserTeamMessage.readyToWalk', index=18,
      number=19, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='naothmessages.BUUserTeamMessage.key', index=19,
      number=100, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("none").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=526,
  serialized_end=1190,
)


_TEAMMESSAGE_DATA = _descriptor.Descriptor(
  name='Data',
  full_name='naothmessages.TeamMessage.Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='playerNum', full_name='naothmessages.TeamMessage.Data.playerNum', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='teamColor', full_name='naothmessages.TeamMessage.Data.teamColor', index=1,
      number=11, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='pose', full_name='naothmessages.TeamMessage.Data.pose', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ballAge', full_name='naothmessages.TeamMessage.Data.ballAge', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ballPosition', full_name='naothmessages.TeamMessage.Data.ballPosition', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ballVelocity', full_name='naothmessages.TeamMessage.Data.ballVelocity', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='fallen', full_name='naothmessages.TeamMessage.Data.fallen', index=6,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user', full_name='naothmessages.TeamMessage.Data.user', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='frameInfo', full_name='naothmessages.TeamMessage.Data.frameInfo', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='teamNumber', full_name='naothmessages.TeamMessage.Data.teamNumber', index=9,
      number=12, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1299,
  serialized_end=1687,
)

_TEAMMESSAGE = _descriptor.Descriptor(
  name='TeamMessage',
  full_name='naothmessages.TeamMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='naothmessages.TeamMessage.data', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='messageDrop', full_name='naothmessages.TeamMessage.messageDrop', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TEAMMESSAGE_DATA, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1193,
  serialized_end=1687,
)

_ROBOTROLE.fields_by_name['role_static'].enum_type = _ROBOTROLESTATIC
_ROBOTROLE.fields_by_name['role_dynamic'].enum_type = _ROBOTROLEDYNAMIC
_OPPONENT.fields_by_name['poseOnField'].message_type = CommonTypes__pb2._POSE2D
_BUUSERTEAMMESSAGE.fields_by_name['opponents'].message_type = _OPPONENT
_BUUSERTEAMMESSAGE.fields_by_name['teamBall'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_BUUSERTEAMMESSAGE.fields_by_name['ntpRequest'].message_type = _NTP
_BUUSERTEAMMESSAGE.fields_by_name['ballVelocity'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_BUUSERTEAMMESSAGE.fields_by_name['robotState'].enum_type = _ROBOTSTATE
_BUUSERTEAMMESSAGE.fields_by_name['robotRole'].message_type = _ROBOTROLE
_TEAMMESSAGE_DATA.fields_by_name['teamColor'].enum_type = _TEAMCOLOR
_TEAMMESSAGE_DATA.fields_by_name['pose'].message_type = CommonTypes__pb2._POSE2D
_TEAMMESSAGE_DATA.fields_by_name['ballPosition'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_TEAMMESSAGE_DATA.fields_by_name['ballVelocity'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_TEAMMESSAGE_DATA.fields_by_name['user'].message_type = _BUUSERTEAMMESSAGE
_TEAMMESSAGE_DATA.fields_by_name['frameInfo'].message_type = Framework__Representations__pb2._FRAMEINFO
_TEAMMESSAGE_DATA.containing_type = _TEAMMESSAGE
_TEAMMESSAGE.fields_by_name['data'].message_type = _TEAMMESSAGE_DATA
_TEAMMESSAGE.fields_by_name['messageDrop'].message_type = _DROPS
DESCRIPTOR.message_types_by_name['RobotRole'] = _ROBOTROLE
DESCRIPTOR.message_types_by_name['Opponent'] = _OPPONENT
DESCRIPTOR.message_types_by_name['Ntp'] = _NTP
DESCRIPTOR.message_types_by_name['Drops'] = _DROPS
DESCRIPTOR.message_types_by_name['BUUserTeamMessage'] = _BUUSERTEAMMESSAGE
DESCRIPTOR.message_types_by_name['TeamMessage'] = _TEAMMESSAGE
DESCRIPTOR.enum_types_by_name['TeamColor'] = _TEAMCOLOR
DESCRIPTOR.enum_types_by_name['RobotState'] = _ROBOTSTATE
DESCRIPTOR.enum_types_by_name['RobotRoleStatic'] = _ROBOTROLESTATIC
DESCRIPTOR.enum_types_by_name['RobotRoleDynamic'] = _ROBOTROLEDYNAMIC
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RobotRole = _reflection.GeneratedProtocolMessageType('RobotRole', (_message.Message,), dict(
  DESCRIPTOR = _ROBOTROLE,
  __module__ = 'TeamMessage_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.RobotRole)
  ))
_sym_db.RegisterMessage(RobotRole)

Opponent = _reflection.GeneratedProtocolMessageType('Opponent', (_message.Message,), dict(
  DESCRIPTOR = _OPPONENT,
  __module__ = 'TeamMessage_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Opponent)
  ))
_sym_db.RegisterMessage(Opponent)

Ntp = _reflection.GeneratedProtocolMessageType('Ntp', (_message.Message,), dict(
  DESCRIPTOR = _NTP,
  __module__ = 'TeamMessage_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Ntp)
  ))
_sym_db.RegisterMessage(Ntp)

Drops = _reflection.GeneratedProtocolMessageType('Drops', (_message.Message,), dict(
  DESCRIPTOR = _DROPS,
  __module__ = 'TeamMessage_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Drops)
  ))
_sym_db.RegisterMessage(Drops)

BUUserTeamMessage = _reflection.GeneratedProtocolMessageType('BUUserTeamMessage', (_message.Message,), dict(
  DESCRIPTOR = _BUUSERTEAMMESSAGE,
  __module__ = 'TeamMessage_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.BUUserTeamMessage)
  ))
_sym_db.RegisterMessage(BUUserTeamMessage)

TeamMessage = _reflection.GeneratedProtocolMessageType('TeamMessage', (_message.Message,), dict(

  Data = _reflection.GeneratedProtocolMessageType('Data', (_message.Message,), dict(
    DESCRIPTOR = _TEAMMESSAGE_DATA,
    __module__ = 'TeamMessage_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.TeamMessage.Data)
    ))
  ,
  DESCRIPTOR = _TEAMMESSAGE,
  __module__ = 'TeamMessage_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.TeamMessage)
  ))
_sym_db.RegisterMessage(TeamMessage)
_sym_db.RegisterMessage(TeamMessage.Data)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\031de.naoth.rc.core.messages'))
_BUUSERTEAMMESSAGE.fields_by_name['isPenalized'].has_options = True
_BUUSERTEAMMESSAGE.fields_by_name['isPenalized']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_BUUSERTEAMMESSAGE.fields_by_name['teamNumber'].has_options = True
_BUUSERTEAMMESSAGE.fields_by_name['teamNumber']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_TEAMMESSAGE_DATA.fields_by_name['teamColor'].has_options = True
_TEAMMESSAGE_DATA.fields_by_name['teamColor']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
# @@protoc_insertion_point(module_scope)



from ._experimental_parser import Parser
from ._experimental_reader import Reader


def get_representations(path_to_file):
    my_parser = Parser()

    logged_representation = set()

    # iterate over the log and put the representation names in a set
    log = Reader(path_to_file, my_parser)
    for idx, frame in enumerate(log):
        names = frame.get_names()
        for name in names:
            logged_representation.add(name)

    return logged_representation



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: AudioData.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='AudioData.proto',
  package='naothmessages',
  syntax='proto2',
  serialized_pb=_b('\n\x0f\x41udioData.proto\x12\rnaothmessages\"l\n\tAudioData\x12\x0f\n\x07samples\x18\x01 \x02(\x0c\x12\x12\n\nsampleSize\x18\x02 \x02(\r\x12\x12\n\nsampleRate\x18\x03 \x02(\r\x12\x13\n\x0bnumChannels\x18\x04 \x02(\r\x12\x11\n\ttimestamp\x18\x05 \x02(\rB\x1b\n\x19\x64\x65.naoth.rc.core.messages')
)




_AUDIODATA = _descriptor.Descriptor(
  name='AudioData',
  full_name='naothmessages.AudioData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='samples', full_name='naothmessages.AudioData.samples', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sampleSize', full_name='naothmessages.AudioData.sampleSize', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sampleRate', full_name='naothmessages.AudioData.sampleRate', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numChannels', full_name='naothmessages.AudioData.numChannels', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='naothmessages.AudioData.timestamp', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=34,
  serialized_end=142,
)

DESCRIPTOR.message_types_by_name['AudioData'] = _AUDIODATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AudioData = _reflection.GeneratedProtocolMessageType('AudioData', (_message.Message,), dict(
  DESCRIPTOR = _AUDIODATA,
  __module__ = 'AudioData_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.AudioData)
  ))
_sym_db.RegisterMessage(AudioData)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\031de.naoth.rc.core.messages'))
# @@protoc_insertion_point(module_scope)



import types
import numpy as np

from ._experimental_parser import Parser
from ._experimental_reader import Reader
from ..constants import Joints
from ..pb import (
    CommonTypes_pb2,
    Framework_Representations_pb2
)

"""
    TODO describe use case
"""


def transform_log_to_hdf5(log_path, representations):
    # init parser
    log_parser = Parser()
    log = Reader(log_path, log_parser)

    # step 1: split frames into their single representations and put them in a array
    splitted_log = {}
    for rep in representations:
        splitted_log[rep] = list([f[rep] for f in log])

    # step 2: flatten the splitted Log
    # each representation is a dict with the members as key
    # each key refers to list of values
    flat_log = flatten(splitted_log)

    return flat_log


def flatten(step):
    return_step = {}
    for rep in list(step.keys()):
        return_step[rep] = flatten_list_of_objects(step[rep])

    return return_step


def flatten_list_of_objects(obj_list):
    return_dict = {}
    # special handling of joint data
    if isinstance(obj_list[0], (Framework_Representations_pb2.JointData,
                                Framework_Representations_pb2.SensorJointData)):
        return flatten_joint_data(obj_list)
    if isinstance(obj_list[0], Framework_Representations_pb2.FSRData):
        return flatten_fsr_data(obj_list)

    # TODO: replace with inspect.getmembers(obj_list, lambda x: not (inspect.ismethod(x) or inspect.isfunction(x) or inspect.isbuiltin(x) or inspect.isclass(x)))
    #       shouldn't use dir function
    member = [f for f in dir(obj_list[0]) if f[0] != '_' and f[0].islower()]
    member = [m for m in member if not m.startswith(("im_", "co_", "func_",))]
    member = [m for m in member if not m.startswith(("denominator", "imag", "numerator", "real"))]
    member = [m for m in member if
              not isinstance(obj_list[0].__getattribute__(m), (types.BuiltinFunctionType, types.MethodType))]

    for m in member:
        if isinstance(obj_list[0].__getattribute__(m), (bool, int, float, complex)):  # check for 'primitive' types
            return_dict[m] = [f.__getattribute__(m) for f in obj_list]
        elif isinstance(obj_list[0].__getattribute__(m), CommonTypes_pb2.Pose3D):
            return_dict[m] = {'rotation': [parse_rotation_matrix(f.__getattribute__(m).rotation) for f in obj_list],
                              'translation': flatten_list_of_objects(
                                  [f.__getattribute__(m).translation for f in obj_list])}
        elif hasattr(obj_list[0].__getattribute__(m), '__len__'):  # it is some kind of list
            if len(obj_list[0].__getattribute__(m)) == 0:  # it's an empty list?
                continue
            # TODO: correct handling for cases like list of vector3d and alike
            return_dict[m] = [[g for g in f.__getattribute__(m)] for f in
                              obj_list]  # assume that this list-like member has a different length in each frame
        else:  # flatten class until primitive...
            return_dict[m] = flatten_list_of_objects([f.__getattribute__(m, ) for f in obj_list])

    return return_dict


def flatten_joint_data(obj_list):
    return_dict = {}

    member = [f for f in dir(obj_list[0]) if f[0] != '_' and f[0].islower()]
    member = [m for m in member if not m.startswith(("im_", "co_", "func_",))]
    member = [m for m in member if not m.startswith(("denominator", "imag", "numerator", "real"))]
    member = [m for m in member if
              not isinstance(obj_list[0].__getattribute__(m), (types.BuiltinFunctionType, types.MethodType))]

    for m in member:
        if isinstance(obj_list[0].__getattribute__(m), Framework_Representations_pb2.JointData):
            return_dict[m] = flatten_joint_data([f.__getattribute__(m, ) for f in obj_list])
        else:
            return_dict[m] = {}
            joint_values = list(zip(*[[g for g in f.__getattribute__(m)] for f in obj_list]))
            for key, value in Joints.items():
                return_dict[m][value] = joint_values[key]

    return return_dict


def flatten_fsr_data(obj_list):
    return_dict = {}

    member = [f for f in dir(obj_list[0]) if f[0] != '_' and f[0].islower()]
    member = [m for m in member if not m.startswith(("im_", "co_", "func_",))]
    member = [m for m in member if not m.startswith(("denominator", "imag", "numerator", "real"))]
    member = [m for m in member if
              not isinstance(obj_list[0].__getattribute__(m), (types.BuiltinFunctionType, types.MethodType))]

    for m in member:
        if isinstance(obj_list[0].__getattribute__(m), (bool, int, float, complex)):  # check for 'primitive' types
            return_dict[m] = [f.__getattribute__(m) for f in obj_list]
        elif hasattr(obj_list[0].__getattribute__(m), '__len__'):  # it is some kind of list
            if len(obj_list[0].__getattribute__(m)) == 0:  # it's an empty list?
                continue
            return_dict[m] = {}
            # assume that this list-like member as the same length in each frame
            # => assume that each entry has a special meaning so rearrange them
            for idx, values in enumerate(zip(*[[g for g in f.__getattribute__(m)] for f in obj_list])):
                return_dict[m][str(idx)] = list(values)
        else:  # flatten class until primitive...
            return_dict[m] = flatten_list_of_objects([f.__getattribute__(m, ) for f in obj_list])

    return return_dict


def parse_rotation_matrix(rot):
    row1 = [rot[0].x, rot[1].x, rot[2].x]
    row2 = [rot[0].y, rot[1].y, rot[2].y]
    row3 = [rot[0].z, rot[1].z, rot[2].z]
    return np.matrix([row1, row2, row3])



import os
import struct as _struct
from typing import Iterator

from ._experimental_parser import Parser

# protobuf
from google.protobuf.message import Message as _Message

"""
NaoTH log file parsing implementation in python 3.

Open a NaoTH log file over the Reader class. For easy cleanup, the use of pythons "with" keyword is supported and 
highly recommended:

# open a log file example
with Reader('naoth_logfile.log') as reader:
    # parse log file frames... 
    # Note: All operations on the reader and its parsed log file frames must be contained in this scope
    
If you don't make use of the "with" keyword, make sure to close the reader by calling reader.close()

Log file frames can be parsed using the reader.read() method. This function implements an iterator returning Frame instances.
For more information on log file frames, see the documentation in the Frame class.

# frame parsing example
for frame in reader.read():
    # retrieve some data from the frame...
"""


class IncompleteFrameException(Exception):
    """
    This exception is raised in Reader.read and Reader.diet_read if an incomplete frame was detected,
    except the reader is instanced with skip_incomplete=True (default)
    """
    pass


class Scanner:
    """
    Interface to the log file handle. Allows scanning the log file for positions of NaoTH representations.
    """

    def __init__(self, log_file):
        self.log_file = log_file
        self._scan_position = 0

    def _read_bytes(self, n):
        """
        Read n bytes from the log file. If there are not enough bytes left an EOFError is raised.
        :param n: number of bytes to read
        :returns byte string of size n
        """
        if n == 0:
            return b''
        data = self.log_file.read(n)
        if len(data) != n:
            raise EOFError()
        return data

    def _read_long(self):
        return _struct.unpack('=l', self._read_bytes(4))[0]

    def _read_char(self):
        char = _struct.unpack('=c', self._read_bytes(1))[0]
        return char.decode("utf-8")

    def _read_string(self):
        """
        Read a '\0' terminated string from the log file.
        """
        chars = []
        c = self._read_char()
        while c != '\0':
            chars.append(c)
            c = self._read_char()
        return ''.join(chars)

    @staticmethod
    def size_of_field(name, payload_size):
        """
        Calculates the byte string size of a NaoTH representation in the log file
        :param name: of member
        :param payload_size
        :returns sizeOf(frame number) + sizeOf(name + '\0') + sizeOf(payload size) + payload size
        """
        return _struct.calcsize('=l') + (len(name) + 1) * _struct.calcsize('=c') + _struct.calcsize('=l') + payload_size

    def set_scan_position(self, frame_start_position):
        """
        Sets the position of the scanner. BEWARE: Must be the start of a NaoTH representation in the log file.
        :param frame_start_position: position in log file where a NaoTH representation starts
        """
        self._scan_position = frame_start_position

    def scan_field(self):
        """
        Reads name, frame number and payload positions of the next NaoTH representation.
        :returns start position, corresponding frame number, payload position (position, size) tuple
        """
        field_start = self._scan_position
        self.log_file.seek(field_start)

        # read message header
        frame_number = self._read_long()

        name = self._read_string()
        message_size = self._read_long()
        data_pos = (self.log_file.tell(), message_size)

        # calculate next field position (skipping payload of previous one)
        self._scan_position = self.log_file.tell() + message_size

        return field_start, frame_number, name, data_pos

    def read_data(self, position, size):
        """
        Reads data bytes from the log file.
        :param position: starting position of the data in the log file
        :param size: size of the data
        :returns data bytes
        """
        if size == 0:
            return b''

        self.log_file.seek(position)
        data = self._read_bytes(size)
        return data


class Frame:
    """
    NaoTH log file frame returned by iterating over the Reader.read method.

    A frame is a collection of logged NaoTH representations with a matching frame number (executed time step).
    NaoTH representations are referred to as (name, field) tuples, where "field" corresponds to the NaoTH representation
    as protobuf Message.

    NaoTH Representations can be accessed and are being parsed by calling the __getitem__ method with its name.
        e.g. frame['FrameInfo'] where 'FrameInfo' is the name of the representation
    """

    def __init__(self, frame_start, frame_number, scanner, parser, cache=False):
        """
        NaoTH log file frame containing NaoTH representations as fields.

        :param frame_start: start of the frame in the log file
        :param frame_number: executed time step
        :param scanner: Scanner of current log file
        :param parser: Parser of current log file
        :param cache: if True cache already parsed members, may increase performance but affects memory usage
        """
        self.scanner = scanner
        self.parser = parser

        # Frame position in the log file
        self.start = frame_start

        # Size of the frame, if it would be converted to bytes using the __bytes__ method:
        # frame.size == len(bytes(frame))
        # Note: When NaoTH representations are added externally (e.g. by calling the add_field function),
        #       the size value does not match the size of the originating log file frame.
        self.size = 0

        self.number = frame_number

        """
        Fields (NaoTH representations) of this frame. Maps name of the NaoTH representation to either
            - a (position, size) tuple of NaoTH representation data in the log file
            - a parsed protobuf class representing the NaoTH representation
        """
        self._fields = {}

        self.cache = cache

    def add_field_position(self, name, position, size):
        """
        Add a NaoTH representation to this frame based on its position and size in the log file.
        This will later be parsed when calling the __getitem__ function with the given name.

        This function is usually only called by the log reader while creating this frame.

        :param name: of the NaoTH representation
        :param position: of payload data in the log file
        :param size: of payload data in the log file
        """
        if name in self._fields:
            raise ValueError(f'Frame already contains member {name}.')

        self._fields[name] = position, size
        self.size += Scanner.size_of_field(name, size)

    def add_field(self, name, _field: _Message):
        """
        Add a parsed or new NaoTH representation in form of a protobuf message to this frame.
        This is useful if we are combining log files, like adding camera images which where logged separately.
        e.g. See combine_logs.py example.

        :param name: of the NaoTH representation
        :param _field: protobuf message instance of the NaoTH representation
        """
        self.size += Scanner.size_of_field(name, _field.ByteSize())
        # add new member
        self._fields[name] = _field

    def remove(self, name):
        """
        Remove a member from this frame.
        :param name: of member
        """
        member = self._fields.pop(name)

        # reduce size of this frame
        if isinstance(member, tuple) and len(member) == 2:
            position, size = member
            self.size -= Scanner.size_of_field(name, size)
        elif isinstance(member, _Message):
            self.size -= Scanner.size_of_field(name, member.ByteSize())
        else:
            raise TypeError(f'Type of member {type(member)} must be a (position, size) tuple or protobuf message!')

    def replace_field(self, name, field: _Message):
        """
        Replace a field of this frame with a protobuf message object.
        :param name: of the NaoTH representation
        :param field: protobuf message instance of the NaoTH representation
        """
        if name not in self._fields:
            raise ValueError(f'Frame does not contain member {name}.')

        # remove old member
        self.remove(name)

        self.add_field(name, field)

    def __contains__(self, name):
        """
        Check if the given name matches a NaoTH representation of this frame.

        # e.g. sanity check before accessing the field 'FrameInfo'
        if 'FrameInfo' in frame:
            frame_info = frame['FrameInfo']

        :param name: of a NaoTH representation
        :returns True if a name corresponding NaoTH representation is part of this frame
        """
        return name in self._fields

    def __getitem__(self, name) -> _Message:
        """
        Parse and return the NaoTH representation corresponding to the given name.
        If caching is enabled, save already parsed members for future access.

        # e.g. retrieve 'FrameInfo' representation
        frame_info = frame['FrameInfo']

        :param name: of a NaoTH representation part of this frame
        :returns NaoTH representation as protobuf Message instance
        """
        message = self._fields[name]
        if isinstance(message, tuple):
            if len(message) != 2:
                raise TypeError(f'Type of field {type(message)} must be a (position, size) tuple'
                                f'to be parsed or a protobuf message!')
            # message is not parsed yet, get payload from log file frame
            position, size = message
            try:
                payload = self.scanner.read_data(position, size)
            except EOFError:
                # raise again with fitting error message
                raise EOFError(f'Failed reading data of field {name}, file ended unexpectedly.')

            # parse payload
            if payload:
                message = self.parser.parse(name, payload)
            else:
                message = None

            if self.cache:
                self._fields[name] = message
            return message
        else:
            return message

    def get_names(self):
        """
        :returns names of NaoTH representations part of this frame
        """
        return self._fields.keys()

    def raw_fields(self):
        """
        This function implements an iterator returning the name and NaoTH representation bytes part of this frame
        by using pythons generator functions.

        This is primarily used to serialize this frame in the __bytes__ method.
        """
        for name, message in self._fields.items():
            if isinstance(message, tuple) and len(message) == 2:
                position, size = message
                # message is not parsed yet, get payload from log file frame
                data = self.scanner.read_data(position, size)
                yield name, data
            elif isinstance(message, _Message):
                yield name, message.SerializeToString()
            else:
                raise TypeError(f'Type of field {type(message)} must be a (position, size) tuple or protobuf message!')

    def __len__(self):
        """
        :returns the size of this frame in bytes
        """
        return self.size

    def copy(self):
        """
        :returns a copy of this frame and all its fields.
        """
        frame = Frame(self.start, self.number, self.scanner, self.parser)
        for name, message in self._fields.items():
            if isinstance(message, tuple) and len(message) == 2:
                position, size = message
                frame.add_field_position(name, position, size)
            elif isinstance(message, _Message):
                message_copy = _Message()
                message_copy.CopyFrom(message)
                frame.add_field(name, message_copy)
            else:
                raise TypeError(f'Type of field {type(message)} must be a (position, size) tuple or protobuf message!')
        return frame

    @staticmethod
    def bytes_from_char(char):
        return _struct.pack('=c', bytes(char, 'utf-8'))

    @staticmethod
    def bytes_from_str(string):
        return b''.join(Frame.bytes_from_char(char) for char in string) + Frame.bytes_from_char('\0')

    @staticmethod
    def long_from_int(integer):
        return _struct.pack('=l', integer)

    def __bytes__(self):
        """
        :returns bytes representation of this frame ready be to written to log file

        # log file write example
        with open('naoth_logfile.log', 'wb') as log_file:
            log_file.write(bytes(frame))
        """
        frame_number = self.long_from_int(self.number)
        _bytes = b''.join(
            frame_number
            + self.bytes_from_str(name)
            + self.long_from_int(len(data))
            + data
            for name, data in self.raw_fields()
        )
        assert self.size == len(_bytes)
        return _bytes


class Reader:
    """
    NaoTH log file reader.

    A Nao running the NaoTH Framework logs the state of its representations for every executed time step.
    Representations include camera images, camera matrix data, motion and sensor data, the behavior state and more.

    This log reader combines logged representations to a frame based on their common executed time steps (frame number).

    Create an instance of this reader with a path to a NaoTH Framework generated log file (see def __init__()).
    "with" statement instantiation is supported and highly encouraged to allow the underlying file handle to be closed.
    Otherwise you have to call the close() function yourself.

    To retrieve the frames of a log file call the read() function of an instanced reader.
    This will return an iterator which can be iterated over returning frames in order of their executed time step:

    for frame in reader.read():
        # do something with the frame
    """

    def __init__(self, path, parser=Parser(), skip_incomplete=True):
        """
        Creates a reader instance for parsing a NaoTH log file.

        :param path: path to a log file generated by the NaoTH framework.
        :param parser: optional Parser argument. If logged representations do not have a matching
                       protobuf implementation a specific Parser to handle those cases must be given.
        :param skip_incomplete: Skip incomplete frames and do not raise an IncompleteFrameException (default: True)
        """
        self.log_file = open(path, 'rb')
        self.log_file_size = os.path.getsize(path)

        self.skip_incomplete = skip_incomplete

        # Used to scan the log file for positions of NaoTH representations and to create an index.
        self.scanner = Scanner(self.log_file)
        # Required for parsing the corresponding protobuf classes based on the retrieved NaoTH representation bytes.
        self.parser = parser

        # Index of frames retrieved from the log file
        self.frames = []

        self.furthest_scan_position = 0
        self.eof_reached = False

    def __enter__(self):
        """
        __enter__ function implemented to support the python "with" keyword.
        """
        return self

    def __exit__(self, _type, value, traceback):
        """
        __exit__ function implemented to support the python "with" keyword.
        """
        self.close()

    def close(self):
        """
        Close the underlying file handle.
        """
        self.log_file.close()

    def index_of(self, frame_number):
        """
        Get log reader index position to the corresponding frame number.

        :param frame_number: number of log file frame
        :return: log reader index or None of the frame doesn't exist
        """
        # TODO: optimize search
        for i, frame in enumerate(self.read()):
            if frame.number == frame_number:
                return i
        else:
            return None

    def read(self, start_idx=0) -> Iterator[Frame]:
        """
        Creates an iterator over the log file frames implemented using pythons generator functions.
        With the "yield" keyword, we return the next Frame instance for the iterator.

        # Example iteration using the read method
        for frame in reader.read():
            # do something with the frame

        :param start_idx: First frame for starting the iterator (default: 0 iterate over all frames).
        :raise IncompleteFrameException if an incomplete frame was detected,
               except the reader is instanced with skip_incomplete=True (default)
        """
        if start_idx < 0:
            raise IndexError(f'Start index <{start_idx}> must be positive!')

        # return frames which were already indexed in self.frames
        while start_idx < len(self.frames):
            yield self.frames[start_idx]
            start_idx += 1

        if self.eof_reached:
            # got nothing left to read
            return

        # read more frames from the log file
        idx = len(self.frames)
        self.scanner.set_scan_position(self.furthest_scan_position)
        frame = None
        while True:
            # read next NaoTH representation from log file
            try:
                start_of_field, current_frame_number, name, payload_pos = self.scanner.scan_field()
            except EOFError:
                # stop reading
                self.eof_reached = True
                break

            # Decide which frame the read NaoTH representation is part of
            if frame is None:
                # create a new frame
                frame = Frame(start_of_field, current_frame_number, self.scanner, self.parser)
            elif current_frame_number != frame.number:
                # frame number does not match, read NaoTH representation is part of a new frame

                # before creating a new frame, finish up the last one and return it
                self.frames.append(frame)
                idx += 1
                # only return frames which come after the given start index
                if start_idx < len(self.frames):
                    yield frame

                    if idx < len(self.frames):
                        # After we yielded, another read iterator was advanced further (has read more frames).
                        # This can happen for example, if the user calls the __getitem__ method for a bigger index.
                        # Return already indexed frames and restart the log file scan at the end of that index.
                        while idx < len(self.frames):
                            yield self.frames[idx]
                            idx += 1
                        if self.eof_reached:
                            return
                        # reset scan
                        frame = None
                        self.scanner.set_scan_position(self.furthest_scan_position)
                        continue

                # create new frame
                frame = Frame(start_of_field, current_frame_number, self.scanner, self.parser)

            # Add the read NaoTH representation to its frame
            position, size = payload_pos
            frame.add_field_position(name, position, size)
            self.furthest_scan_position = position + size

        # End of file is reached, return the last log file frame
        if frame is not None and frame.get_names():
            # check if the last frame is complete
            if frame.start + len(frame) <= self.log_file_size:
                self.frames.append(frame)

                # only return frame if it comes after the given start index
                if start_idx < len(self.frames):
                    yield frame
            elif not self.skip_incomplete:
                raise IncompleteFrameException(f'Last log file frame is incomplete, missing '
                                               f'{frame.start + len(frame) - self.log_file_size} bytes.')

    def diet_read(self) -> Iterator[Frame]:
        """
        Same as "read", but does not create a frame index and preserves memory (len(self.frames) will stay 0).
        Useful if the log file is very large and only a single pass through is required.

        :raise IncompleteFrameException if an incomplete frame was detected,
               except the reader is instanced with skip_incomplete=True (default)
        """
        # read frames from the log file
        self.scanner.set_scan_position(0)
        frame = None
        while True:
            # read next NaoTH representation from log file
            try:
                start_of_field, current_frame_number, name, payload_pos = self.scanner.scan_field()
            except EOFError:
                # stop reading
                break

            # Decide which frame the read NaoTH representation is part of
            if frame is None:
                # create a new frame
                frame = Frame(start_of_field, current_frame_number, self.scanner, self.parser)
            elif current_frame_number != frame.number:
                # frame number does not match, read NaoTH representation is part of a new frame

                # before creating a new frame, finish up the last one and return it
                yield frame

                # create new frame
                frame = Frame(start_of_field, current_frame_number, self.scanner, self.parser)

            # Add the read NaoTH representation to its frame
            position, size = payload_pos
            frame.add_field_position(name, position, size)
            self.scanner.set_scan_position(position + size)

        # End of file is reached, return the last log file frame
        if frame is not None and frame.get_names():
            # check if the last frame is complete
            if frame.start + len(frame) <= self.log_file_size:
                yield frame
            elif not self.skip_incomplete:
                raise IncompleteFrameException(f'Last log file frame is incomplete, missing '
                                               f'{frame.start + len(frame) - self.log_file_size} bytes.')

    def __getitem__(self, i):
        """
        Get ith frame from the log file.

        # e.g.
        3rd_frame = reader[2]
        """
        if i >= len(self.frames):
            # frame i was not indexed yet, read more frames
            try:
                return next(self.read(start_idx=i))
            except StopIteration:
                raise IndexError
        return self.frames[i]

    def file_path(self):
        """
        :returns underlying log file path
        """
        return self.log_file.name
